from __future__ import annotations

from typing import TYPE_CHECKING
from typing import Any
from typing import Sequence
from typing import TypeVar

from prompt_toolkit import Application  # pyright: ignore
from prompt_toolkit.application.current import get_app  # pyright: ignore
from prompt_toolkit.filters import Condition  # pyright: ignore
from prompt_toolkit.filters import FilterOrBool  # pyright: ignore
from prompt_toolkit.key_binding import KeyBindings  # pyright: ignore
from prompt_toolkit.key_binding import merge_key_bindings  # pyright: ignore
from prompt_toolkit.key_binding.bindings.focus import focus_next  # pyright: ignore
from prompt_toolkit.key_binding.bindings.focus import focus_previous  # pyright: ignore
from prompt_toolkit.key_binding.defaults import load_key_bindings  # pyright: ignore
from prompt_toolkit.layout import ConditionalMargin  # pyright: ignore
from prompt_toolkit.layout import Dimension as D  # pyright: ignore
from prompt_toolkit.layout import HSplit  # pyright: ignore
from prompt_toolkit.layout import Layout  # pyright: ignore
from prompt_toolkit.layout import ScrollbarMargin  # pyright: ignore
from prompt_toolkit.layout import Window  # pyright: ignore
from prompt_toolkit.layout.controls import FormattedTextControl  # pyright: ignore
from prompt_toolkit.styles import BaseStyle  # pyright: ignore
from prompt_toolkit.styles import Style  # pyright: ignore
from prompt_toolkit.widgets import Button  # pyright: ignore
from prompt_toolkit.widgets import Dialog  # pyright: ignore
from prompt_toolkit.widgets import Label  # pyright: ignore
from prompt_toolkit.widgets import RadioList
from prompt_toolkit.widgets import TextArea
from prompt_toolkit.widgets.toolbars import ValidationToolbar

from ozi_core.ui._style import _style_dict
from ozi_core.ui.menu import MenuButton

if TYPE_CHECKING:  # pragma: no cover
    from prompt_toolkit.buffer import Buffer  # pyright: ignore
    from prompt_toolkit.completion import Completer  # pyright: ignore
    from prompt_toolkit.formatted_text import AnyFormattedText  # pyright: ignore
    from prompt_toolkit.key_binding.key_processor import KeyPressEvent  # pyright: ignore
    from prompt_toolkit.validation import Validator  # pyright: ignore


_T = TypeVar('_T')


class Admonition(RadioList[_T]):
    """Simple scrolling text dialog."""

    open_character = ''
    close_character = ''
    container_style = 'class:admonition-list'
    default_style = 'class:admonition'
    selected_style = 'class:admonition-selected'
    checked_style = 'class:admonition-checked'
    multiple_selection = False

    def __init__(  # noqa: C901
        self: Admonition,
        values: Sequence[tuple[_T, Any]],
        default: _T | None = None,
    ) -> None:  # pragma: no cover
        super().__init__(values, default)
        kb = KeyBindings()

        @kb.add('pageup')
        def _pageup(event: KeyPressEvent) -> None:
            w = event.app.layout.current_window
            if w.render_info:
                self._selected_index = max(
                    0,
                    self._selected_index - len(w.render_info.displayed_lines),
                )

        @kb.add('pagedown')
        def _pagedown(event: KeyPressEvent) -> None:
            w = event.app.layout.current_window
            if w.render_info:
                self._selected_index = min(
                    len(self.values) - 1,
                    self._selected_index + len(w.render_info.displayed_lines),
                )

        @kb.add('up')
        def _up(event: KeyPressEvent) -> None:
            _pageup(event)

        @kb.add('down')
        def _down(event: KeyPressEvent) -> None:
            _pagedown(event)

        @kb.add('enter')
        @kb.add(' ')
        def _click(event: KeyPressEvent) -> None:
            self._handle_enter()

        self.control = FormattedTextControl(
            self._get_text_fragments,
            key_bindings=kb,
            focusable=True,
        )

        self.window = Window(
            content=self.control,
            style=self.container_style,
            right_margins=[
                ConditionalMargin(
                    margin=ScrollbarMargin(display_arrows=True),
                    filter=Condition(lambda: self.show_scrollbar),
                ),
            ],
            dont_extend_height=True,
            wrap_lines=True,
            always_hide_cursor=True,
        )

    def _handle_enter(self) -> None:  # noqa: DC103,ANN101,RUF100
        pass  # pragma: no cover


def _return_none() -> None:  # pragma: no cover
    """Button handler that returns None."""
    get_app().exit()


def admonition_dialog(
    title: str = '',
    text: str = '',
    heading_label: str = '',
    ok_text: str | None = None,
    cancel_text: str | None = None,
    style: BaseStyle | None = None,
) -> Application[list[Any]]:  # pragma: no cover
    """Admonition dialog shortcut.
    The focus can be moved between the list and the Ok/Cancel button with tab.
    """
    if ok_text is None:
        ok_text = MenuButton.OK._str
    if cancel_text is None:
        cancel_text = MenuButton.EXIT._str

    if style is None:
        style_dict = _style_dict
        style_dict.update(
            {
                'dialog.body admonition-list': '#e1e7ef',
                'dialog.body admonition': '#e1e7ef',
                'dialog.body admonition-selected': '#030711',
                'dialog.body admonition-checked': '#030711',
            },
        )
        style = Style.from_dict(style_dict)

    def ok_handler() -> None:
        get_app().exit(result=True)

    lines = text.splitlines()

    cb_list = Admonition(values=list(zip(lines, lines)), default=None)
    longest_line = len(max(lines, key=len))
    dialog = Dialog(
        title=title,
        body=HSplit(
            [Label(text=heading_label, dont_extend_height=True), cb_list],
            padding=1,
        ),
        buttons=[
            Button(text=ok_text, handler=ok_handler),
            Button(text=cancel_text, handler=_return_none),
        ],
        with_background=True,
        width=min(max(longest_line + 8, 40), 80),
    )
    bindings = KeyBindings()
    bindings.add('tab')(focus_next)
    bindings.add('s-tab')(focus_previous)

    return Application(
        layout=Layout(dialog),
        key_bindings=merge_key_bindings([load_key_bindings(), bindings]),
        mouse_support=True,
        style=style,
        full_screen=True,
    )


def input_dialog(  # pragma: no cover
    title: AnyFormattedText = '',
    text: AnyFormattedText = '',
    ok_text: str | None = None,
    cancel_text: str | None = None,
    completer: Completer | None = None,
    validator: Validator | None = None,
    password: FilterOrBool = False,
    style: BaseStyle | None = None,
    multiline: bool = False,
    default: str = '',
) -> Application[str]:
    """
    Display a text input box.
    Return the given text, or None when cancelled.
    """
    if ok_text is None:
        ok_text = MenuButton.OK._str
    if cancel_text is None:
        cancel_text = MenuButton.BACK._str

    def accept(buf: Buffer) -> bool:
        get_app().layout.focus(ok_button)
        return True  # Keep text.

    def ok_handler() -> None:
        get_app().exit(result=textfield.text)

    ok_button = Button(text=ok_text, handler=ok_handler)
    cancel_button = Button(text=cancel_text, handler=_return_none)
    lines = default.splitlines()
    longest_line = len(max(lines, key=len)) if len(lines) > 0 else 40
    textfield = TextArea(
        text=default,
        multiline=multiline,
        password=password,
        completer=completer,
        validator=validator,
        accept_handler=accept,
        height=max(len(lines), 1),
        width=min(max(longest_line + 8, 40), 80),
    )

    dialog = Dialog(
        title=title,
        body=HSplit(
            [
                Label(text=text, dont_extend_height=True),
                textfield,
                ValidationToolbar(),
            ],
            padding=D(preferred=1, max=1),
        ),
        buttons=[ok_button, cancel_button],
        with_background=True,
    )
    bindings = KeyBindings()
    bindings.add('tab')(focus_next)
    bindings.add('s-tab')(focus_previous)

    return Application(
        layout=Layout(dialog),
        key_bindings=merge_key_bindings([load_key_bindings(), bindings]),
        mouse_support=True,
        style=style,
        full_screen=True,
    )



"""
This type stub file was generated by pyright.
"""

from typing import TYPE_CHECKING
from typing import Any

"""Internationalization utilities."""
if TYPE_CHECKING:
    ...
_LOCALE = ...

class Translation:
    __slots__ = ('_locale', 'data')
    def __init__(self) -> None: ...
    @property
    def locale(self) -> str | Any: ...
    @locale.setter
    def locale(self, loc: str) -> None: ...
    def __call__(self, _key: str, **kwargs: str) -> str: ...

TRANSLATION: Translation = ...



# ozi/fix/parser.py
# Part of the OZI Project, under the Apache License v2.0 with LLVM Exceptions.
# See LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
"""``ozi-fix`` console application."""
import sys
from argparse import SUPPRESS
from argparse import ArgumentParser
from argparse import BooleanOptionalAction

from ozi_core._i18n import TRANSLATION
from ozi_core.fix.validate import AppendRewriteCommandTarget
from ozi_core.new.defaults import COPYRIGHT_HEAD

parser = ArgumentParser(
    prog='ozi-fix',
    description=sys.modules[__name__].__doc__,
    add_help=False,
    usage=f"""%(prog)s [{TRANSLATION('term-options')}] | [{TRANSLATION('term-positional-args')}]

{TRANSLATION('adm-disclaimer-text')}
""",
)
parser.add_argument(
    '--add',
    metavar='FILENAME',
    nargs='?',
    action=AppendRewriteCommandTarget,
    default=['ozi.phony'],
    help=SUPPRESS,
)
parser.add_argument(
    '--remove',
    metavar='FILENAME',
    nargs='?',
    action=AppendRewriteCommandTarget,
    default=['ozi.phony'],
    help=SUPPRESS,
)
parser.add_argument(
    '--strict',
    default=False,
    action=BooleanOptionalAction,
    help=SUPPRESS,
)
parser.add_argument(
    '--pretty',
    default=False,
    action=BooleanOptionalAction,
    help=SUPPRESS,
)
subparser = parser.add_subparsers(help='', metavar='', dest='fix')
helpers = parser.add_mutually_exclusive_group()
helpers.add_argument('-h', '--help', action='help', help=TRANSLATION('term-help-help'))
missing_parser = subparser.add_parser(
    'missing',
    prog='ozi-fix missing',
    aliases=['m', 'mis'],
    usage=f'%(prog)s [{TRANSLATION("term-options")}] [{TRANSLATION("term-output")}] target',
    allow_abbrev=True,
    help=TRANSLATION('term-help-fix-missing'),
)
missing_parser.add_argument(
    '--add',
    metavar='FILENAME',
    nargs='?',
    action=AppendRewriteCommandTarget,
    default=['ozi.phony'],
    help=SUPPRESS,
)
missing_parser.add_argument(
    '--remove',
    metavar='FILENAME',
    nargs='?',
    action=AppendRewriteCommandTarget,
    default=['ozi.phony'],
    help=SUPPRESS,
)
missing_output = missing_parser.add_argument_group(TRANSLATION('term-output'))
missing_output.add_argument(
    '--strict',
    default=False,
    action=BooleanOptionalAction,
    help=TRANSLATION('term-help-strict'),
)
missing_output.add_argument(
    '--pretty',
    default=False,
    action=BooleanOptionalAction,
    help=TRANSLATION('term-help-pretty'),
)
missing_parser.add_argument(
    'target',
    type=str,
    nargs='?',
    default='.',
    help=TRANSLATION('term-help-fix-target'),
)
source_parser = subparser.add_parser(
    'source',
    aliases=['s', 'src'],
    prog='ozi-fix source',
    usage=f'%(prog)s [{TRANSLATION("term-options")}] [{TRANSLATION("term-output")}] target',
    allow_abbrev=True,
    help=TRANSLATION('term-help-fix-source'),
)
test_parser = subparser.add_parser(
    'test',
    prog='ozi-fix test',
    usage=f'%(prog)s [{TRANSLATION("term-options")}] [{TRANSLATION("term-output")}] target',
    aliases=['t', 'tests'],
    allow_abbrev=True,
    help=TRANSLATION('term-help-fix-test'),
)
source_parser.add_argument(
    '-a',
    '--add',
    metavar='FILENAME',
    nargs='?',
    action=AppendRewriteCommandTarget,
    default=['ozi.phony'],
    help=TRANSLATION('term-help-fix-add'),
)
source_parser.add_argument(
    '-r',
    '--remove',
    metavar='FILENAME',
    nargs='?',
    action=AppendRewriteCommandTarget,
    default=['ozi.phony'],
    help=TRANSLATION('term-help-fix-remove'),
)
source_parser.add_argument(
    '-c',
    '--copyright-head',
    metavar='HEADER',
    type=str,
    default=COPYRIGHT_HEAD,
    help=TRANSLATION('term-copyright-head'),
)
source_output = source_parser.add_argument_group(TRANSLATION('term-output'))
source_output.add_argument(
    '--strict',
    default=False,
    action=BooleanOptionalAction,
    help=TRANSLATION('term-help-strict'),
)
source_output.add_argument(
    '--pretty',
    default=False,
    action=BooleanOptionalAction,
    help=TRANSLATION('term-help-pretty'),
)
source_output.add_argument(
    '--interactive-io',
    default=False,
    action=BooleanOptionalAction,
    help=SUPPRESS,
)
source_parser.add_argument(
    'target',
    type=str,
    nargs='?',
    default='.',
    help=TRANSLATION('term-help-fix-target'),
)
test_parser.add_argument(
    '-a',
    '--add',
    metavar='FILENAME',
    nargs='?',
    action=AppendRewriteCommandTarget,
    default=['ozi.phony'],
    help=TRANSLATION('term-help-fix-add'),
)
test_parser.add_argument(
    '-r',
    '--remove',
    metavar='FILENAME',
    nargs='?',
    action=AppendRewriteCommandTarget,
    default=['ozi.phony'],
    help=TRANSLATION('term-help-fix-remove'),
)
test_parser.add_argument(
    '-c',
    '--copyright-head',
    metavar='HEADER',
    type=str,
    default=COPYRIGHT_HEAD,
    help=TRANSLATION('term-copyright-head'),
)
test_output = test_parser.add_argument_group(TRANSLATION('term-output'))
test_output.add_argument(
    '--strict',
    default=False,
    action=BooleanOptionalAction,
    help=TRANSLATION('term-help-strict'),
)
test_output.add_argument(
    '--pretty',
    default=False,
    action=BooleanOptionalAction,
    help=TRANSLATION('term-help-pretty'),
)
test_output.add_argument(
    '--interactive-io',
    default=False,
    action=BooleanOptionalAction,
    help=SUPPRESS,
)
test_parser.add_argument(
    'target',
    type=str,
    nargs='?',
    default='.',
    help=TRANSLATION('term-help-fix-target'),
)
interactive_parser = subparser.add_parser(
    'interactive',
    prog='ozi-fix interactive',
    aliases=['i'],
    usage='%(prog)s target',
    allow_abbrev=True,
    help=TRANSLATION('term-help-fix-interactive'),
)
interactive_parser.add_argument(
    'target',
    type=str,
    nargs='?',
    default='.',
    help=TRANSLATION('term-help-fix-target'),
)



"""
This type stub file was generated by pyright.
"""

from __future__ import annotations

import sys
from argparse import Namespace
from typing import TYPE_CHECKING

"""
``ozi-new`` interactive prompts
"""
if sys.platform != 'win32':
    ...
else:
    ...
if TYPE_CHECKING:
    ...

def interactive_prompt(project: Namespace) -> list[str]: ...



from __future__ import annotations

from argparse import Action
from argparse import ArgumentParser
from argparse import Namespace
from typing import TYPE_CHECKING
from typing import Any
from typing import Sequence

from prompt_toolkit.validation import ValidationError  # pyright: ignore
from prompt_toolkit.validation import Validator

from ozi_core._i18n import TRANSLATION

if TYPE_CHECKING:
    from prompt_toolkit.document import Document  # pyright: ignore


def _copy_items(items: list[str] | None) -> list[str]:  # pragma: defer to python
    """Copied from the argparse module."""
    if items is None:
        return []
    # The copy module is used only in the 'append' and 'append_const'
    # actions, and it is needed only when the default value isn't a list.
    # Delay its import for speeding up the common case.
    if type(items) is list:
        return items[:]
    import copy

    return copy.copy(items)


def valid_rewrite_command_target(target: str) -> bool:  # pragma: defer to E2E
    return len(target.rstrip('/').split('/')) == 1


class AppendRewriteCommandTarget(Action):
    def __call__(  # pragma: defer to E2E
        self: AppendRewriteCommandTarget,
        parser: ArgumentParser,
        namespace: Namespace,
        values: str | Sequence[Any] | None,
        option_string: str | None = None,
    ) -> None:
        if not valid_rewrite_command_target(values):  # type: ignore
            parser.error(f'Nested paths are not yet supported. Path: {values}')
        items = getattr(namespace, self.dest, None)
        items = _copy_items(items)
        items.append(values)  # type: ignore
        setattr(namespace, self.dest, [values])


class RewriteCommandTargetValidator(Validator):
    """Validate that a target name is valid."""

    def validate(  # pragma: defer to E2E
        self,  # noqa: ANN101,RUF100
        document: Document,
    ) -> None:  # pragma: no cover
        if len(document.text) == 0:
            raise ValidationError(0, TRANSLATION('err-no-empty'))
        if not valid_rewrite_command_target(document.text):
            raise ValidationError(
                len(document.text),
                TRANSLATION('err-no-nested-fix-support'),
            )



# ozi/fix/missing.py
# Part of the OZI Project, under the Apache License v2.0 with LLVM Exceptions.
# See LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
"""Find missing OZI project files."""
from __future__ import annotations

import re
import sys
from email import message_from_string
from pathlib import Path
from typing import TYPE_CHECKING

from ozi_spec import METADATA
from ozi_templates.filter import underscorify  # pyright: ignore
from tap_producer import TAP

from ozi_core._i18n import TRANSLATION
from ozi_core.fix.build_definition import walk
from ozi_core.meson import load_ast
from ozi_core.meson import project_metadata
from ozi_core.pkg_extra import parse_extra_pkg_info

if sys.version_info >= (3, 11):  # pragma: no cover
    import tomllib as toml
elif sys.version_info < (3, 11):  # pragma: no cover
    import tomli as toml

if TYPE_CHECKING:  # pragma: no cover
    from email.message import Message

PKG_INFO = """Metadata-Version: @METADATA_VERSION@
Name: @PROJECT_NAME@
Version: @SCM_VERSION@
License: @LICENSE@
@REQUIRED@
@REQUIREMENTS_IN@

@README_TEXT@
"""

readme_ext_to_content_type = {
    '.rst': 'text/x-rst',
    '.md': 'text/markdown',
    '.txt': 'text/plain',
    '': 'text/plain',
}


def get_relpath_expected_files(
    kind: str,
    name: str,
) -> tuple[Path, tuple[str, ...] | tuple[()]]:
    match kind:
        case 'test':
            rel_path = Path('tests')
            expected_files = METADATA.spec.python.src.required.test
        case 'root':
            rel_path = Path('.')
            expected_files = METADATA.spec.python.src.required.root
        case 'source':
            rel_path = Path(underscorify(name).lower())
            expected_files = METADATA.spec.python.src.required.source
        case _:  # pragma: no cover
            rel_path = Path('.')
            expected_files = ()
    return rel_path, expected_files


def render_requirements(target: Path) -> str:
    """Render requirements.in as it would appear in PKG-INFO"""
    try:
        requirements = (  # pragma: no cover
            r.partition('\u0023')[0]
            for r in filter(
                lambda r: not (r.startswith('\u0023') or r == '\n'),
                target.joinpath('requirements.in').read_text('utf-8').splitlines(),
            )
        )
    except FileNotFoundError:
        with target.joinpath('pyproject.toml').open('rb') as f:
            requirements = toml.load(f).get('project', {}).get('dependencies', [])
    return ''.join([f'Requires-Dist: {req}\n' for req in requirements])


def render_pkg_info(target: Path, name: str, _license: str) -> Message:  # noqa: C901
    """Render PKG-INFO as it would be produced during packaging."""
    with target.joinpath('pyproject.toml').open('rb') as f:
        pyproject = toml.load(f)
        setuptools_scm = pyproject.get('tool', {}).get('setuptools_scm', {})
        ozi_build = pyproject.get('tool', {}).get('ozi-build', {}).get('metadata', {})
        if setuptools_scm.get('version_file_template'):  # pragma: no cover
            TAP.ok('setuptools_scm', 'PKG-INFO', 'template')
            return message_from_string(
                setuptools_scm.get('version_file_template', '@README_TEXT@')
                .replace(
                    '@README_TEXT@',
                    target.joinpath('README').read_text(),
                )
                .replace('@PROJECT_NAME@', name)
                .replace('@LICENSE@', _license)
                .replace('@REQUIREMENTS_IN@\n', render_requirements(target))
                .replace('@SCM_VERSION@', '{version}'),
            )
        else:
            required = ''
            for key in ('home-page', 'summary', 'author', 'author-email', 'keywords'):
                val = ozi_build.get(key, None)
                if val is not None:  # pragma: no cover
                    required += f'{key.capitalize()}: {val}\n'
            for ext in ('.rst', '.txt', '.md'):
                readme = target.joinpath(f'README{ext}')
                if readme.exists():
                    required += (
                        f'Description-Content-Type: {readme_ext_to_content_type.get(ext)}\n'
                    )
            required += ''.join(
                [f'Classifier: {req}\n' for req in ozi_build.get('classifiers', [])],
            )
            msg = (
                PKG_INFO.replace('@LICENSE@', _license)
                .replace('@REQUIREMENTS_IN@', render_requirements(target).strip())
                .replace('@SCM_VERSION@', '{version}')
                .replace('@PROJECT_NAME@', name)
                .replace('@METADATA_VERSION@', METADATA.spec.python.support.metadata_version)
                .replace('@REQUIRED@', required.strip('\n'))
                .replace('@README_TEXT@', target.joinpath('README').read_text())
            )
            return message_from_string(msg)


def python_support(pkg_info: Message) -> set[tuple[str, str]]:
    """Check PKG-INFO Message for python support."""
    remaining_pkg_info = {
        (k, v)
        for k, v in pkg_info.items()
        if k not in METADATA.spec.python.pkg.info.required
    }
    for k, v in iter(METADATA.ozi.python_support.classifiers[:4]):
        if (k, v) in remaining_pkg_info:
            TAP.ok(k, v)
        else:
            TAP.not_ok(TRANSLATION('term-missing'), v)  # pragma: no cover
    return remaining_pkg_info


def required_extra_pkg_info(pkg_info: Message) -> dict[str, str]:
    """Check missing required OZI extra PKG-INFO"""
    remaining_pkg_info = python_support(pkg_info)
    remaining_pkg_info.difference_update(set(iter(METADATA.ozi.python_support.classifiers)))
    for k, v in iter(remaining_pkg_info):
        TAP.ok(k, v)
    extra_pkg_info, errstr = parse_extra_pkg_info(pkg_info)
    if errstr not in ('', None):
        TAP.not_ok(TRANSLATION('term-missing'), str(errstr))  # pragma: no cover
    for k, v in extra_pkg_info.items():
        TAP.ok(k, v)
    return extra_pkg_info


def required_pkg_info(
    target: Path,
) -> tuple[str, dict[str, str]]:
    """Find missing required PKG-INFO"""
    ast = load_ast(str(target))
    name = ''
    license_ = ''
    if ast:
        name, license_ = project_metadata(ast)
    pkg_info = render_pkg_info(target, name, license_)
    for i in METADATA.spec.python.pkg.info.required:
        v = pkg_info.get(i, None)
        if v is not None:
            TAP.ok(i, v)
        else:  # pragma: no cover
            TAP.not_ok(TRANSLATION('term-missing'), i)
    extra_pkg_info = required_extra_pkg_info(pkg_info)
    name = re.sub(r'[-_.]+', '-', pkg_info.get('Name', '')).lower()
    return name, extra_pkg_info


def required_files(
    kind: str,
    target: Path,
    name: str,
) -> list[str]:
    """Count missing files required by OZI"""
    found_files = []
    rel_path, expected_files = get_relpath_expected_files(kind, name)
    for file in expected_files:
        f = rel_path / file
        if not target.joinpath(f).exists():  # pragma: no cover
            TAP.not_ok(TRANSLATION('term-missing'), str(f))
            continue  # pragma: defer to https://github.com/nedbat/coveragepy/issues/198
        TAP.ok(str(f))
        found_files.append(file)
    list(
        walk(
            target,
            rel_path,
            found_files=found_files,
            project_name=underscorify(name).lower(),
        ),
    )
    return found_files


def report(
    target: Path,
) -> tuple[str, Message | None, list[str], list[str], list[str]]:
    """Report missing OZI project files
    :param target: Relative path to target directory.
    :return: Normalized Name, PKG-INFO, found_root, found_sources, found_tests
    """
    target = Path(target)
    name = None
    pkg_info = None
    extra_pkg_info: dict[str, str] = {}
    try:
        name, extra_pkg_info = required_pkg_info(target)
    except FileNotFoundError:
        name = ''
        TAP.not_ok(TRANSLATION('term-missing'), TRANSLATION('term-required-metadata'))
    found_source_files = required_files(
        'source',
        target,
        name,
    )
    found_test_files = required_files(
        'test',
        target,
        name,
    )
    found_root_files = required_files(
        'root',
        target,
        name,
    )
    all_files = (  # pragma: defer to TAP-Consumer
        ['PKG-INFO'],
        extra_pkg_info,
        found_root_files,
        found_source_files,
        found_test_files,
    )
    try:  # pragma: defer to TAP-Consumer
        sum(map(len, all_files))
    except TypeError:  # pragma: defer to TAP-Consumer
        TAP.bail_out(TRANSLATION('term-missing'))
    return (  # pragma: defer to TAP-Consumer
        name,
        pkg_info,
        found_root_files,
        found_source_files,
        found_test_files,
    )



"""
This type stub file was generated by pyright.
"""

from functools import lru_cache
from typing import TypeAlias

from mesonbuild.mparser import ArrayNode
from mesonbuild.mparser import AssignmentNode
from mesonbuild.mparser import BaseNode
from mesonbuild.mparser import CodeBlockNode
from mesonbuild.mparser import DictNode
from mesonbuild.mparser import ElementaryNode
from mesonbuild.mparser import ForeachClauseNode
from mesonbuild.mparser import FunctionNode
from mesonbuild.mparser import IdNode
from mesonbuild.mparser import MethodNode
from mesonbuild.mparser import NotNode
from mesonbuild.mparser import PlusAssignmentNode
from mesonbuild.mparser import UMinusNode

"""A query tool for meson.build files."""

SelectValue: TypeAlias = type[AssignmentNode | PlusAssignmentNode | NotNode | UMinusNode]
SelectItems: TypeAlias = type[ForeachClauseNode]
WhereValue: TypeAlias = type[ArrayNode | DictNode | MethodNode | FunctionNode]
WhereItems: TypeAlias = type[IdNode]

def load_ast(source_root: str) -> CodeBlockNode | None:
    """Load the :abbr:`AST (Abstract Syntax Tree)` from the root :file:`meson.build`.

    :param source_root: Directory containing a top-level :file:`meson.build`.
    :type source_root: str
    :return: The AST for a meson build definition if one is available OR None.
    :rtype: CodeBlockNode | None
    """

def project_metadata(ast: CodeBlockNode) -> tuple[str, str]:
    """Extract project metadata from :file:`meson.build`

    :param ast: The AST for a :file:`meson.build`.
    :type ast: CodeBlockNode
    :return: The project name and license identifier.
    :rtype: tuple[str, str]
    """

@lru_cache(typed=True)
def query_simple(
    ast: CodeBlockNode, select: SelectValue = ..., where: WhereValue = ...
) -> set[BaseNode]:
    """Run a simplistic query with no node filtering.

    :param ast: The AST for a :file:`meson.build`.
    :type ast: CodeBlockNode
    :param select: Select node type, defaults to AssignmentNode
    :type select: SelectValue, optional
    :param where: Where node type, defaults to ArrayNode
    :type where: WhereValue, optional
    :return: Set of all nodes matching the selector.
    :rtype: set[BaseNode]
    """

@lru_cache(typed=True)
def query_complex(
    ast: CodeBlockNode,
    select: SelectValue = ...,
    where: WhereValue = ...,
    get: type[ElementaryNode] = ...,
) -> set[str]:
    """Run a complex (filtered) query of build item values.

    :param ast: The AST for a :file:`meson.build`.
    :type ast: CodeBlockNode
    :param select: select type, defaults to AssignmentNode
    :type select: SelectValue, optional
    :param where: where type, defaults to ArrayNode
    :type where: WhereValue, optional
    :param get: get type, defaults to StringNode
    :type get: type[ElementaryNode], optional
    :return: A set of node values matching the selector
    :rtype: set[str]
    """

@lru_cache(typed=True)
def query_loop_assignments(
    ast: CodeBlockNode,
    select: SelectItems = ...,
    where: type[ElementaryNode] = ...,
    get: type[ElementaryNode] = ...,
) -> set[str]:
    """Get a set of selected array items (must be assigned to a variable).

    :param ast: The AST for a :file:`meson.build`.
    :type ast: CodeBlockNode
    :param select: Select node type, defaults to ForeachClauseNode
    :type select: SelectItems, optional
    :param where: Where node type, defaults to IdNode
    :type where: WhereItems, optional
    :param get: Query node type, defaults to StringNode
    :type get: type[ElementaryNode], optional
    :return: A set of query matches
    :rtype: set[str]
    """

@lru_cache(typed=True)
def find_var_suffix(ast: CodeBlockNode, var_suffix: str) -> set[str]:
    """Get a set of build items based on a variable name suffix.

    :param ast: The AST for a :file:`meson.build`.
    :type ast: CodeBlockNode
    :param var_suffix: The text to look for.
    :type var_suffix: str
    :return: A set of query matches
    :rtype: set[str] | None
    """

def query_build_value(source_root: str, query: str) -> bool | None:
    """Load a :file:`meson.build` project and check if a query exists in array assignments.

    :param source_root: The path to directory containing a :file:`meson.build`
    :type source_root: str
    :param query: The text to look for.
    :type query: str
    :return: True if a query match is found, False if not, and None if the
             :file:`meson.build` could not be loaded.
    :rtype: set[str] | None
    """

def get_items_by_suffix(source_root: str, query: str) -> set[str] | None:
    """Load a :file:`meson.build` project and return build items.

    :param source_root: The path to directory containing a :file:`meson.build`
    :type source_root: str
    :param query: The text to look for.
    :type query: str
    """



"""
This type stub file was generated by pyright.
"""

from typing import Any

def main_menu(
    project: Any, output: dict[str, list[str]], prefix: dict[str, str]
) -> tuple[None | list[str] | bool, dict[str, list[str]], dict[str, str]]: ...
def options_menu(
    project: Any, output: dict[str, list[str]], prefix: dict[str, str]
) -> tuple[None | list[str] | bool, dict[str, list[str]], dict[str, str]]: ...
def edit_menu(
    project: Any, output: dict[str, list[str]], prefix: dict[str, str]
) -> tuple[None | list[str] | bool, dict[str, list[str]], dict[str, str]]: ...



from _typeshed import Incomplete
from dataclasses import dataclass
from ozi_core import __version__ as __version__

core_version: Incomplete
HEADER: Incomplete

@dataclass(kw_only=True)
class OziInteractiveConfig:
    """General config options for dialog-based CLI."""

    language: str | None = ...

@dataclass(kw_only=True)
class OziFixConfig:
    """Persistent ``ozi-fix interactive`` settings."""

    copyright_head: str | None = ...
    pretty: bool | None = ...
    strict: bool | None = ...

@dataclass(kw_only=True)
class OziNewConfig:
    """Persistent ``ozi-new interactive`` settings."""

    allow_file: list[str] | None = ...
    author: str | None = ...
    author_email: str | None = ...
    ci_provider: str | None = ...
    check_package_exists: bool | None = ...
    copyright_head: str | None = ...
    enable_cython: bool | None = ...
    enable_uv: bool | None = ...
    github_harden_runner: bool | None = ...
    maintainer: str | None = ...
    maintainer_email: str | None = ...
    language: list[str] | None = ...
    readme_type: str | None = ...
    strict: bool | None = ...
    verify_email: bool | None = ...

@dataclass(kw_only=True)
class OziConfig:
    """Persistent ``ozi-* interactive`` settings."""

    fix: OziFixConfig = ...
    new: OziNewConfig = ...
    interactive: OziInteractiveConfig = ...

def read_user_config() -> OziConfig: ...
def write_user_config(data: OziConfig) -> None: ...
